/*
 * REPOWEB, repository manager.
 *
 * Terms of license - http://opensource.org/licenses/apachepl.php
 */
package idea.jalopy;

import org.apache.log4j.AppenderSkeleton;
import org.apache.log4j.Level;
import org.apache.log4j.spi.LoggingEvent;

import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.vfs.VirtualFile;

import de.hunsicker.io.FileFormat;

import de.hunsicker.jalopy.Jalopy;
import de.hunsicker.jalopy.storage.Convention;
import de.hunsicker.jalopy.storage.ConventionDefaults;
import de.hunsicker.jalopy.storage.ConventionKeys;
import de.hunsicker.jalopy.storage.History;
import de.hunsicker.jalopy.storage.Loggers;
import de.hunsicker.jalopy.swing.SettingsDialog;

import java.awt.*;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;


/**
 * Singleton for jalopy stuff.
 */
class JalopyPlugin
{
    private static JalopyPlugin ourInstance   = null;
    private Jalopy              jalopy;
    private Logger              logger        = Logger.getInstance ( "idea.jalopy.JalopyPlugin" );
    private ReportBuilder       reportBuilder = new ReportBuilder(  );

    private JalopyPlugin (  )
    {
        if ( logger.isDebugEnabled (  ) )
        {
            logger.debug ( "JalopyPlugin is up" );
        }
    }

    public static void init (  )
    {
        if ( ourInstance != null )
        {
            return;
        }

        ourInstance = new JalopyPlugin(  );
    }

    public static void dispose (  )
    {
        ourInstance = null;
    }

    public static JalopyPlugin getInstance (  )
    {
        return ourInstance;
    }

    public JalopyReport format ( VirtualFile ideaFile )
                         throws FileNotFoundException
    {
        if ( logger.isDebugEnabled (  ) )
        {
            logger.debug ( "format" );
        }

        initEngine (  );

        File file = new File( ideaFile.getPath (  ) );
        jalopy.setOutput ( file );
        jalopy.setInput ( file );

        reportBuilder.record (  );

        jalopy.format (  );

        return reportBuilder.createReport ( ideaFile );
    }

    public JalopyReport format ( String text )
    {
        if ( logger.isDebugEnabled (  ) )
        {
            logger.debug ( "format" );
        }

        initEngine (  );

        StringWriter writer = new StringWriter( text.length (  ) );
        jalopy.setOutput ( writer );

        jalopy.setFileFormat ( getFileFormat ( text ) );
        jalopy.setInput ( text, "unused" );

        reportBuilder.record (  );
        jalopy.format (  );

        return reportBuilder.createReport ( writer.toString (  ) );
    }

    private FileFormat getFileFormat ( String input )
    {
        BufferedReader bufferedReader = new BufferedReader( new StringReader( input ) );
        String         line           = null;

        try
        {
            line = bufferedReader.readLine (  );
        }
        catch ( IOException e )
        {
            throw new RuntimeException( e );
        }

        String EOL = input.substring ( line.length (  ), line.length (  ) + 2 );

        if ( EOL.startsWith ( "\n" ) )
        {
            return FileFormat.valueOf ( "\n" );
        }

        if ( EOL.startsWith ( "\r\n" ) )
        {
            return FileFormat.valueOf ( "\r\n" );
        }

        return null;
    }

    public static void displayJalopyPref ( Container parent )
    {
        SettingsDialog dialog = SettingsDialog.create ( findWindow ( parent ) );
        dialog.pack (  );
        dialog.setLocationRelativeTo ( parent );
        dialog.setVisible ( true );
    }

    private static Window findWindow ( Container parent )
    {
        if ( ( parent == null ) || parent instanceof Window )
        {
            return ( Window ) parent;
        }

        return findWindow ( parent.getParent (  ) );
    }

    private void initEngine (  )
    {
        if ( this.jalopy == null )
        {
            this.jalopy = new Jalopy(  );
            Loggers.ALL.addAppender ( reportBuilder );
        }

        jalopy.reset (  );
        configureJalopy (  );
    }

    private void configureJalopy (  )
    {
        Convention settings    = Convention.getInstance (  );
        int        backupLevel = settings.getInt ( ConventionKeys.BACKUP_LEVEL, ConventionDefaults.BACKUP_LEVEL );
        jalopy.setBackup ( backupLevel > 0 );
        jalopy.setBackupDirectory ( settings.get ( ConventionKeys.BACKUP_DIRECTORY, Convention.getBackupDirectory (  ).getAbsolutePath (  ) ) );
        jalopy.setHistoryPolicy ( History.Policy.valueOf ( settings.get ( ConventionKeys.HISTORY_POLICY, ConventionDefaults.HISTORY_POLICY ) ) );
        jalopy.setHistoryMethod ( History.Method.valueOf ( settings.get ( ConventionKeys.HISTORY_METHOD, ConventionDefaults.HISTORY_METHOD ) ) );
        jalopy.setInspect ( settings.getBoolean ( ConventionKeys.INSPECTOR, ConventionDefaults.INSPECTOR ) );
        jalopy.setBackupLevel ( backupLevel );
        jalopy.setFileFormat ( FileFormat.AUTO );
        jalopy.setForce ( settings.getBoolean ( ConventionKeys.FORCE_FORMATTING, ConventionDefaults.FORCE_FORMATTING ) );
    }

    /**
     * Collect warning and error for the report built after format.
     */
    private final class ReportBuilder
        extends AppenderSkeleton
    {
        JalopyReport report;

        ReportBuilder (  )
        {
            this.name = "JalopyForIdeaSpyAppender";
        }

        public void append ( LoggingEvent ev )
        {
            if ( report == null )
            {
                return;
            }

            int i = ev.getLevel (  ).toInt (  );

            if ( ( i == Level.WARN_INT ) || ( i == Level.ERROR_INT ) || ( i == Level.FATAL_INT ) )
            {
                report.addEvent ( ev );
            }
        }

        public void close (  )
        {
        }

        public synchronized void doAppend ( LoggingEvent ev )
        {
            append ( ev );
        }

        public boolean requiresLayout (  )
        {
            return false;
        }

        boolean checkEntryConditions (  )
        {
            return true;
        }

        public void record (  )
        {
            report = new JalopyReport(  );
        }

        public JalopyReport createReport ( String result )
        {
            finishReport (  );
            report.setResult ( result );

            JalopyReport tmp = report;
            report = null;

            return tmp;
        }

        public JalopyReport createReport ( VirtualFile ideaFile )
        {
            finishReport (  );
            report.setFormatedFile ( ideaFile );

            JalopyReport tmp = report;
            report = null;

            return tmp;
        }

        private void finishReport (  )
        {
            report.setState ( jalopy.getState (  ) );

            if ( jalopy.getState (  ) == Jalopy.State.OK )
            {
                logger.debug ( "  Successfully formatted" );
            }
            else if ( jalopy.getState (  ) == Jalopy.State.WARN )
            {
                logger.debug ( "  Formatted with warnings" );
            }
            else if ( jalopy.getState (  ) == Jalopy.State.ERROR )
            {
                logger.debug ( "  Could not be formatted" );
            }
        }
    }
}
