/*
 * REPOWEB, repository manager.
 *
 * Terms of license - http://opensource.org/licenses/apachepl.php
 */
package idea.jalopy;

import com.intellij.openapi.actionSystem.DataContext;
import com.intellij.openapi.editor.Document;
import com.intellij.openapi.editor.Editor;
import com.intellij.openapi.editor.SelectionModel;
import com.intellij.openapi.editor.actionSystem.EditorWriteActionHandler;
import com.intellij.openapi.localVcs.LvcsAction;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.Messages;

import com.intellij.psi.PsiDocumentManager;
import com.intellij.psi.PsiFile;
import com.intellij.psi.codeStyle.CodeStyleManager;

import com.intellij.util.IncorrectOperationException;

import java.io.PrintWriter;
import java.io.StringWriter;

import javax.swing.*;


/**
 * Launch format (jalopy or Idea default one).
 */
final class OpenedFileHandler
    extends EditorWriteActionHandler
{
    public void executeWriteAction ( Editor      editor,
                                     DataContext dataContext )
    {
        // Don't do anything on read-only documents
        final Document document = editor.getDocument (  );
        final Project  project  = editor.getProject (  );
        PsiDocumentManager.getInstance ( project ).doPostponedOperationsAndUnblockDocument ( document );

        if ( ! document.isWritable (  ) || ! IdeaUtil.isInProject ( IdeaUtil.getCurrentFile ( dataContext ), dataContext ) )
        {
            return;
        }

        try
        {
            if ( IdeaUtil.isJavaFile ( dataContext, editor ) )
            {
                jalopyFormat ( editor, dataContext );
            }
            else
            {
                ideaFormat ( editor, IdeaUtil.getProject ( dataContext ) );
            }
        }
        catch ( Exception e )
        {
            Messages.showMessageDialog ( errorToString ( e ), "Erreur !", new ImageIcon( getClass (  ).getResource ( "/jalopy.png" ) ) );
        }
    }

    private String errorToString ( Exception error )
    {
        StringWriter writer = new StringWriter(  );
        error.printStackTrace ( new PrintWriter( writer ) );

        return writer.toString (  );
    }

    public JalopyReport jalopyFormat ( Editor      editor,
                                       DataContext dataContext )
    {
        final Document document = editor.getDocument (  );

        if ( ! document.isWritable (  ) )
        {
            return new JalopyReport(  );
        }

        LvcsAction action = IdeaUtil.lvcsStart ( dataContext, "Jalopy format" );

        try
        {
            if ( JalopyComponent.isOptimizeimport (  ) )
            {
                optimizeImports ( editor, dataContext );
            }

            final String before = document.getText (  );
            JalopyReport report = JalopyPlugin.getInstance (  ).format ( before );

            if ( ! report.hasError (  ) && ! before.equals ( report.getResult (  ) ) )
            {
                PsiDocumentManager.getInstance ( editor.getProject (  ) ).doPostponedOperationsAndUnblockDocument ( document );
                //                document.replaceString ( 0, document.getTextLength (  ), report.getResult (  ) );
                document.setText ( report.getResult (  ) );
            }

            if ( report.hasError (  ) )
            {
                Messages.showErrorDialog ( report.buildMessage (  ), report.stateToString (  ) );
            }
            else if ( report.hasWarning (  ) )
            {
                Messages.showWarningDialog ( report.buildMessage (  ), report.stateToString (  ) );
            }

            return report;
        }
        finally
        {
            action.finish (  );
        }
    }

    private void ideaFormat ( Editor  editor,
                              Project project )
    {
        if ( editor.getSelectionModel (  ).hasSelection (  ) )
        {
            ideaFormatRange ( editor, project );
        }
        else
        {
            ideaFormatFile ( editor, project );
        }
    }

    private void ideaFormatFile ( Editor  editor,
                                  Project project )
    {
        try
        {
            CodeStyleManager.getInstance ( project ).reformat ( IdeaUtil.getPsiFile ( editor, project ) );
        }
        catch ( IncorrectOperationException e )
        {
            throw new RuntimeException( e );
        }
    }

    private void ideaFormatRange ( Editor  editor,
                                   Project project )
    {
        final SelectionModel selectionModel = editor.getSelectionModel (  );
        int                  selectionStart = selectionModel.getSelectionStart (  );
        int                  selectionEnd   = selectionModel.getSelectionEnd (  );

        try
        {
            CodeStyleManager.getInstance ( project ).reformatRange ( IdeaUtil.getPsiFile ( editor, project ), selectionStart, selectionEnd );
        }
        catch ( IncorrectOperationException e )
        {
            throw new RuntimeException( e );
        }
    }

    private void optimizeImports ( Editor      editor,
                                   DataContext dataContext )
    {
        final Project    project          = IdeaUtil.getProject ( dataContext );
        PsiFile          psiFile          = IdeaUtil.getPsiFile ( editor, project );
        CodeStyleManager codeStyleManager = CodeStyleManager.getInstance ( project );

        try
        {
            codeStyleManager.optimizeImports ( psiFile );
        }
        catch ( IncorrectOperationException e )
        {
            throw new RuntimeException( e );
        }
    }
}
